@@ -73,7 +73,6 @@
 #define   MCHelpSwEP2        "\n  ep2           Expand paths to full"
 #define   MCHelpSwEP3        "\n  ep3           Expand paths to full including the drive letter"
 #define   MCHelpSwF          "\n  f             Freshen files"
-#define   MCHelpSwFCU        "\n  fcu[file]     Read archive comment from Unicode file"
 #define   MCHelpSwHP         "\n  hp[password]  Encrypt both file data and headers"
 #define   MCHelpSwIDP        "\n  id[c,d,p,q]   Disable messages"
 #define   MCHelpSwIEML       "\n  ieml[addr]    Send archive by email"
@@ -108,6 +107,7 @@
 #define   MCHelpSwRV         "\n  rv[N]         Create recovery volumes"
 #define   MCHelpSwS          "\n  s[<N>,v[-],e] Create solid archive"
 #define   MCHelpSwSm         "\n  s-            Disable solid archiving"
+#define   MCHelpSwSC         "\n  sc<chr><obj>  Specify the character set"
 #define   MCHelpSwSFX        "\n  sfx[name]     Create SFX archive"
 #define   MCHelpSwSI         "\n  si[name]      Read data from standard input (stdin)"
 #define   MCHelpSwSL         "\n  sl<size>      Process files with size less than specified"
@@ -133,7 +133,7 @@
 #define   MCHelpSwXa         "\n  x@            Read file names to exclude from stdin"
 #define   MCHelpSwXal        "\n  x@<list>      Exclude files in specified list file"
 #define   MCHelpSwY          "\n  y             Assume Yes on all queries"
-#define   MCHelpSwZ          "\n  z<file>       Read archive comment from file"
+#define   MCHelpSwZ          "\n  z[file]       Read archive comment from file"
 #define   MBadArc            "\nERROR: Bad archive %s\n"
 #define   MAskPsw            "Enter password (will not be echoed)"
 #define   MAskPswEcho        "Enter password"
@@ -345,3 +345,4 @@
 #define   MRecVolLimit       "\nTotal number of usual and recovery volumes must not exceed 255"
 #define   MVolumeNumber      "volume %d"
 #define   MCannotDelete      "\nCannot delete %s"
+#define   MCalcCRC           "\nCalculating the control sum"
