@@ -15,6 +15,7 @@ ScanTree::ScanTree(StringList *FileMasks,int Recurse,bool GetLinks,int GetDirs)
   Errors=0;
   FastFindFile=false;
   *ErrArcName=0;
+  Cmd=NULL;
 }
 
 
@@ -58,6 +59,8 @@ bool ScanTree::PrepareMasks()
 {
   if (!FileMasks->GetString(CurMask,CurMaskW,sizeof(CurMask)))
     return(false);
+  CurMask[ASIZE(CurMask)-1]=0;
+  CurMaskW[ASIZE(CurMaskW)-1]=0;
 #ifdef _WIN_32
   UnixSlashToDos(CurMask);
 #endif
@@ -138,6 +141,8 @@ int ScanTree::FindProc(FindData *FindData)
       FastFindFile=true;
       if (!FindCode)
       {
+        if (Cmd!=NULL && Cmd->ExclCheck(CurMask,true))
+          return(SCAN_NEXT);
         ErrHandler.OpenErrorMsg(ErrArcName,CurMask);
         return(FindData->Error ? SCAN_ERROR:SCAN_NEXT);
       }
@@ -153,6 +158,9 @@ int ScanTree::FindProc(FindData *FindData)
       Error=false;
 #endif
 
+    if (Cmd!=NULL && Cmd->ExclCheck(CurMask,true))
+      Error=false;
+
 #ifndef SILENT
     if (Error)
     {
